mod components;
use std::time::Duration;

use axum::{extract::State, response::IntoResponse};
use axum_controller::*;
use datastar::{
    prelude::{FragmentMergeMode, MergeFragments},
    Sse,
};
use hypertext::{maud, GlobalAttributes, Renderable};

use crate::{ui::components::*, AppState};

pub mod html_elements {
    use hypertext::elements;
    pub use hypertext::html_elements::*;

    elements! {
        bla {
            blub
        }
        my_element {
            my_attribute
        }
    }
}

pub struct UiController {}

#[controller(
    state = AppState
)]
impl UiController {
    #[route(GET "/msg")]
    async fn message(State(_): State<AppState>) -> impl IntoResponse {
        tokio::time::sleep(Duration::from_millis(500)).await;

        let sse_stream: Sse<_> = Sse(async_stream::stream! {
            yield MergeFragments::new(maud! { div #user-1 .mb-2 { strong { "You:" } "userInput" } }.render())
                // .id("chat")
                .selector("#chat")
                .merge_mode(FragmentMergeMode::Append)
                .into();

            tokio::time::sleep(Duration::from_millis(500)).await;

            yield MergeFragments::new(maud! { div #llm-1 class="mb-2" { strong { "LLM:" } "This is a mock response."} }.render())
                // .id("chat")
                .selector("#chat")
                .merge_mode(FragmentMergeMode::Append)
                .into();
        });

        sse_stream
    }

    #[route(GET "/")]
    async fn index(State(_): State<AppState>) -> impl IntoResponse {
        main_page(maud! {
                    div class="container p-4 mx-auto" {
                        h1 class="mb-4 text-2xl font-bold" {
                            "LLM Chat App"
                        }
                        div .bg-white  .p-6 .rounded-lg .shadow-md {
                            div #chat .mb-4 {
                                // Chat messages will appear here
                            }
                            form id="chat-form" {
                                textarea id="user-input" class="p-2 mb-2 w-full rounded-lg border" placeholder="Type your message..." {}
                                button type="button" data-on-click="@get('/msg')" class="p-2 text-white bg-blue-500 rounded-lg" {
                                    "Send"
                                }
                            }
                        }
                    }
                }
            )
        .render()
    }
}
